using System;
using System.Net.Sockets;
using System.Threading;
using System.Windows.Forms;

namespace Server
{

	/// <summary>
	/// Component for handling messages from clients. This is in a separate 
	/// thread because we need to allow the election server manager and 
	/// client requests to run in parallel. Network object listens for 
	/// incoming requests and creates separate <c>Worker</c> threads that
	/// handle the rest of the communication.
	/// </summary>
	public class Network
	{
		Server server;
		TcpListener listener;
  
		/// <summary>
		/// Constructor initializes the Network object
		/// </summary>
		/// <param name="server">Contains the pointer to the <c>Server</c></param>
		/// <param name="listener">Contains the pointer to the listener of 
		/// the servers side</param>
		public Network(Server server, TcpListener listener)
		{
			this.server = server;
			this.listener = listener;
		}
			
		/// <summary>
		/// Method that listens to the incoming connections from the client 
		/// and spuns off <c>Worker</c> threads.
		/// </summary>
		/// <remarks>The thread is terminated by the main thread by calling Abort()
		/// on it. It results in the exception that is caught here.</remarks>
		public void run() 
		{
			try 
			{
				while(true) 
				{
					//Pending() method is non-blocking and the thread goes 
					//to sleep for the specified amount of time.
					if (listener.Pending())
					{
						Socket client = listener.AcceptSocket();
						Worker worker = new Worker(server, client);
						Thread t = new Thread(new ThreadStart(worker.run));
						t.Start();  
					}
					else
						Thread.Sleep(1000);

				}
			}
			catch (ThreadAbortException) 
			{
				Application.Exit();
				return;
			}
			catch (VotingAppException e) 
			{
				MessageBox.Show(e.Message + ": " + e.StackTrace, "Fatal Voting Error");
				Application.Exit();
				return;
			}
		}

	}
}


